//
//  ASServerController.h
//  ASServer
//
//  Created by Marian Goldeen on 2/15/05.
//  Copyright 2005 Marian Goldeen. All rights reserved.
//

#import <Foundation/Foundation.h>
#define AS_PORT 8619

@class ASGameServer;
@class ASScoreKeeper;
@class BGWord;

@protocol ASServerProtocol
- (oneway void)helloWorld;
- (oneway void)toggleGameRequestedBy:(bycopy NSString *)playerKey;
- (oneway void)startGameWithBoardRepresentation:(bycopy NSDictionary *)rep requestedBy:(bycopy NSString *)playerKey;
- (oneway void)registerPlayer:(byref id)remote withName:(bycopy NSString *)name;
- (oneway void)removePlayerNamed:(bycopy NSString *)name;
- (oneway void)addWord:(bycopy NSDictionary/*NSString*/ *)word forPlayerNamed:(bycopy NSString *)playerKey;
- (oneway void)removeWordString:(bycopy NSString *)word;
- (oneway void)newTimeAllowed:(int)newTime;
- (oneway void)toggleTimerRequestedByPlayerNamed:(bycopy NSString *)playerKey;
- (oneway void)resumeGameRequestedBy:(bycopy NSString *)playerKey;
- (oneway void)pauseGameRequestedBy:(bycopy NSString *)playerKey;
- (oneway void)resumeGame;
- (oneway void)broadcastMessage:(bycopy NSString *)message fromPlayerNamed:(bycopy NSString *)playerKey;
//- (BOOL)enforceSolitaireGame:(BOOL)flag;
//- (oneway void)playerNamed:(bycopy NSString *)playerName accepted:(BOOL)accepted;
- (oneway void)listen;
- (oneway void)stopListening;
- (oneway void)resetGame;
@end

//! to remove
@protocol ASServerControlProtocol
- (oneway void)stopListening;
- (oneway void)broadcast;
@end

typedef enum _ASGameType {
	ASSolitaire, 
	ASMultiplayer, 
	ASSolitaireEnforced
} ASGameType;

@protocol ASClientProtocol
- (oneway void)beep;
- (oneway void)setGameType:(ASGameType)theType;
- (oneway void)setAllowedStateOfWord:(bycopy NSString *)word to:(BOOL)flag;
- (oneway void)resetScore;
- (oneway void)closeGame;
- (oneway void)fold;
- (oneway void)foldWithMessage:(bycopy NSString *)message;
- (oneway void)startNewGameWithTrayRepresentation:(bycopy NSDictionary *)rep timeAllowed:(int)minutes;
- (oneway void)countdownMinutes:(int)minutes seconds:(int)seconds;
- (oneway void)gameEnded;
- (oneway void)packedFoundWords:(bycopy NSArray *)pack forPlayerNamed:(bycopy NSString *)playerName;
- (oneway void)newTimeAllowed:(int)newTime;
- (oneway void)setMultiplicity:(int)multiplicity;
//- (oneway void)permissionToJoinGameRequestedByPlayerNamed:(bycopy NSString *)playerName;
- (oneway void)addedPlayerNamed:(bycopy NSString *)playerName;
- (oneway void)removedPlayerNamed:(bycopy NSString *)playerName;
- (oneway void)serverRequestedRemoveWordString:(bycopy NSString *)word;
- (oneway void)updateScores:(bycopy NSArray *)changedScores forPlayersWithNames:(bycopy NSArray *)changedPlayers;
- (oneway void)pauseGameRequestedBy:(bycopy NSString *)playerName;
- (oneway void)hideBoard;
- (oneway void)resumeGame;
- (oneway void)revealBoard;
- (oneway void)setTimerButtonForPause;
- (oneway void)setTimerButtonForEnd;
- (bycopy NSString *)playerCannotResumeGameRequestedBy:(bycopy NSString *)playerName;
- (oneway void)playersCannotResumeGame:(bycopy NSArray *)refusingPlayers;
- (oneway void)displayMessage:(bycopy NSString *)message fromPlayerNamed:(bycopy NSString *)playerKey;
@end;

@interface ASServerController : NSObject {
	ASGameServer *_gameServer;
	ASScoreKeeper *_scoreKeeper;
	
	NSSocketPort *_serverSocket;
	NSPort *_controlPort;	//! no longer used
	NSNetService *_service;
	NSConnection *_connection, 
		*_controlConnection, //! no longer used
		*_localClientConnection;
	NSLock *_publishLock,		// used for communication: locked when the server is publishing
		*_threadStartedLock,		// used for communication: locked while separate thread is running
		*_controlConnectionLock; // used as a lock: locked until local connection is set up
	NSAutoreleasePool *_threadPool;
	NSThread *_gameThread; // to keep track---checked in threadWillExit:
	int _port;
}

- (NSString *)serverName;
- (BOOL)setup;
- (void)setupConnection;

- (void)run:(NSDictionary *)portInfo;

- (oneway void)listen;
- (oneway void)stopListening;
- (oneway void)resetGame;
- (oneway void)broadcast;

- (NSLock *)publishLock;
- (NSLock *)threadStartedLock;
- (NSLock *)controlConnectionLock;

- (oneway void)helloWorld;
- (oneway void)toggleGameRequestedBy:(bycopy NSString *)playerKey;
- (oneway void)startGameWithBoardRepresentation:(bycopy NSDictionary *)rep requestedBy:(bycopy NSString *)playerKey;
- (oneway void)registerPlayer:(byref id)remote withName:(bycopy NSString *)name;
- (oneway void)addWord:(bycopy NSDictionary/*NSString*/ *)word forPlayerNamed:(bycopy NSString *)playerKey;
- (oneway void)removeWordString:(bycopy NSString *)word;
- (oneway void)newTimeAllowed:(int)newTime;
- (oneway void)toggleTimerRequestedByPlayerNamed:(bycopy NSString *)playerKey;
- (oneway void)resumeGameRequestedBy:(bycopy NSString *)playerKey;
- (oneway void)pauseGameRequestedBy:(bycopy NSString *)playerKey;
- (oneway void)resumeGame;
- (oneway void)broadcastMessage:(bycopy NSString *)message fromPlayerNamed:(bycopy NSString *)playerKey;
//- (BOOL)enforceSolitaireGame:(BOOL)flag;
//- (oneway void)playerNamed:(bycopy NSString *)playerName accepted:(BOOL)accepted;

- (ASGameServer *)gameServer;
- (ASScoreKeeper *)scoreKeeper;

- (void)threadExitNotification: (NSNotification *)notif;
@end
